@@ -26,8 +26,10 @@ from math import degrees as degrees
 from math import pi as pi
 from math import tan as tan
 import sys
+
 # Third party imports
 import ephem
+
 # Local application imports
 from pyalmanac.lib import config as config
 
@@ -62,6 +64,34 @@ def hhmm(date):
     #       flipped into the next day, however this is not required here.
     return time
 
+def hhmmss(date):       # used by eventtables.py
+    # turn an ephem.date (float) into a time string formatted hh:mm:ss
+    tup = date.tuple()
+    hr = tup[-3]
+    mi = tup[-2]
+    sec = int(round(tup[-1]))
+#    nextday = False
+    if sec == 60:
+        sec =0
+        mi += 1
+    if mi == 60:
+        mi = 0
+        hr += 1
+        if hr == 24:
+            hr = 0
+#            nextday = True	# time rounded up into next day
+    time = '{:02d}:{:02d}:{:02d}'.format(hr,mi,sec)    # time = "%02d:%02d:%02d" %(hr,min,sec)
+#    return time, nextday
+    # NOTE: this function could easily return the information that rounding
+    #       flipped into the next day, however this is not required here.
+    return time
+
+def date2time(date, withseconds = False):
+    if withseconds:
+        return hhmmss(date)
+    else:
+        return hhmm(date)
+
 def nadeg(rad, fixedwidth=1):
     # changes ephem.angle (rad) to the format usually used in the nautical almanac (ddd°mm.m) and returns a string object.
 	# the optional argument specifies the minimum width for degrees (only)
@@ -123,9 +153,9 @@ def sunmoon(date):          # used in suntab(m), sunmoontab(m)
     degm = ephem_moon.g_dec
     decm = nadeg(degm,2)
     
-    #calculate the moons horizontal paralax
+    #calculate the moon's horizontal paralax
     deg = ephem.degrees(ephem_moon.radius/0.272805950305)
-    hp = "{:0.1f}'".format(deg*360*30/ephem.pi)
+    hp = "{:0.1f}'".format(deg*360*30/pi)
     
     #calculate v and d by advancing the time with one hour.
     ephem_moon.compute(date-0.5*ephem.hour,epoch=date-0.5*ephem.hour)
@@ -135,15 +165,18 @@ def sunmoon(date):          # used in suntab(m), sunmoontab(m)
     ephem_moon.compute(date+0.5*ephem.hour,epoch=date+0.5*ephem.hour)
     obs.date = date + 0.5 * ephem.hour
     rghap = ephem.degrees(obs.sidereal_time()-ephem_moon.g_ra).norm
+
     deg = ephem.degrees(ephem.degrees(rghap-rgha).norm-ephem.degrees('14:19:00'))
-    vm = "{:0.1f}'".format(deg*360*30/ephem.pi)
+    vmf = deg*360*30/pi
+    vm = "{:0.1f}'".format(vmf)
+
     deg = ephem.degrees(ephem_moon.g_dec-rdec)
-    dm = "{:0.1f}'".format(deg*360*30/ephem.pi)
+    dmf = deg*360*30/pi
+    dm = "{:0.1f}'".format(dmf)
     
     # degs, degm have been added for the sunmooontab function
     return ghas,decs,gham,vm,decm,dm,hp,degs,degm
 
-##NEW##
 def sun_moon_SD(date):      # used in suntab(m), sunmoontab(m)
     obs = ephem.Observer()
     obs.date = date
@@ -155,13 +188,13 @@ def sun_moon_SD(date):      # used in suntab(m), sunmoontab(m)
     ephem_sun.compute(date+ephem.hour)
     obs.date = date+ephem.hour
     deg = ephem.degrees(ephem_sun.g_dec-dec)
-    ds = "{:0.1f}".format(deg*360*30/ephem.pi)
-    sds = "{:0.1f}".format(ephem_sun.radius*360*30/ephem.pi)
+    ds = "{:0.1f}".format(deg*360*30/pi)
+    sds = "{:0.1f}".format(ephem_sun.radius*360*30/pi)
     
     #Moon
     # compute semi-diameter of moon (in minutes)
     ephem_moon.compute(date)
-    sdm = "{:0.1f}".format(ephem_moon.radius*360*30/ephem.pi)
+    sdm = "{:0.1f}".format(ephem_moon.radius*360*30/pi)
     
     return ds,sds,sdm
 
@@ -232,9 +265,9 @@ def vdm_planets(date):      # used in planetstab(m)
     obs.date = date+ephem.hour
     ghap = ephem.degrees(obs.sidereal_time()-ephem_venus.g_ra).norm
     deg = ephem.degrees(ghap-gha).norm-ephem.degrees('15:00:00')
-    vvenus = "{:0.1f}".format(deg*360*30/ephem.pi)
+    vvenus = "{:0.1f}".format(deg*360*30/pi)
     deg = ephem.degrees(ephem_venus.g_dec-dec)
-    dvenus = "{:0.1f}".format(deg*360*30/ephem.pi)
+    dvenus = "{:0.1f}".format(deg*360*30/pi)
     mvenus = "{:0.1f}".format(ephem_venus.mag)
     
     #Mars
@@ -246,9 +279,9 @@ def vdm_planets(date):      # used in planetstab(m)
     obs.date = date+ephem.hour
     ghap = ephem.degrees(obs.sidereal_time()-ephem_mars.g_ra).norm
     deg = ephem.degrees(ephem.degrees(ghap-gha).norm-ephem.degrees('15:00:00'))
-    vmars = "{:0.1f}".format(deg*360*30/ephem.pi)
+    vmars = "{:0.1f}".format(deg*360*30/pi)
     deg = ephem.degrees(ephem_mars.g_dec-dec)
-    dmars = "{:0.1f}".format(deg*360*30/ephem.pi)
+    dmars = "{:0.1f}".format(deg*360*30/pi)
     mmars = "{:0.1f}".format(ephem_mars.mag)
     
     #Jupiter
@@ -260,9 +293,9 @@ def vdm_planets(date):      # used in planetstab(m)
     obs.date = date+ephem.hour
     ghap = ephem.degrees(obs.sidereal_time()-ephem_jupiter.g_ra).norm
     deg = ephem.degrees(ephem.degrees(ghap-gha).norm-ephem.degrees('15:00:00'))
-    vjup = "{:0.1f}".format(deg*360*30/ephem.pi)
+    vjup = "{:0.1f}".format(deg*360*30/pi)
     deg = ephem.degrees(ephem_jupiter.g_dec-dec)
-    djup = "{:0.1f}".format(deg*360*30/ephem.pi)
+    djup = "{:0.1f}".format(deg*360*30/pi)
     mjup = "{:0.1f}".format(ephem_jupiter.mag)
     
     #Saturn
@@ -274,9 +307,9 @@ def vdm_planets(date):      # used in planetstab(m)
     obs.date = date+ephem.hour
     ghap = ephem.degrees(obs.sidereal_time()-ephem_saturn.g_ra).norm
     deg = ephem.degrees(ephem.degrees(ghap-gha).norm-ephem.degrees('15:00:00'))
-    vsat = "{:0.1f}".format(deg*360*30/ephem.pi)
+    vsat = "{:0.1f}".format(deg*360*30/pi)
     deg = ephem.degrees(ephem_saturn.g_dec-dec)
-    dsat = "{:0.1f}".format(deg*360*30/ephem.pi)
+    dsat = "{:0.1f}".format(deg*360*30/pi)
     msat = "{:0.1f}".format(ephem_saturn.mag)
     
     return vvenus,dvenus,mvenus,vmars,dmars,mmars,vjup,djup,mjup,vsat,dsat,msat
@@ -298,32 +331,32 @@ def ariestransit(date):     # used in planetstab(m)
         hhmm = '0' + hhmm[0:4]
     return hhmm
     
-def planetstransit(date):   # used in starstab
+def planetstransit(date, round2seconds = False):   # used in starstab
     #returns SHA and meridian passage for the navigational planets
 
     obs = ephem.Observer()
     
     obs.date = date
     ephem_venus.compute(date)
-    vsha = nadeg(2*pi-ephem.degrees(ephem_venus.g_ra).norm)
-    vtrans = hhmm(obs.next_transit(ephem_venus))
+    vsha = nadeg(2*pi - ephem.degrees(ephem_venus.g_ra).norm)
+    vtrans = date2time(obs.next_transit(ephem_venus), round2seconds)
     hpvenus = "{:0.1f}".format((tan(6371/(ephem_venus.earth_distance*149597870.7)))*60*180/pi)
     
     obs.date = date
     ephem_mars.compute(date)
-    marssha = nadeg(2*pi-ephem.degrees(ephem_mars.g_ra).norm)
-    marstrans = hhmm(obs.next_transit(ephem_mars))
+    marssha = nadeg(2*pi - ephem.degrees(ephem_mars.g_ra).norm)
+    marstrans = date2time(obs.next_transit(ephem_mars), round2seconds)
     hpmars = "{:0.1f}".format((tan(6371/(ephem_mars.earth_distance*149597870.7)))*60*180/pi)
 
     obs.date = date
     ephem_jupiter.compute(date)
-    jsha = nadeg(2*pi-ephem.degrees(ephem_jupiter.g_ra).norm)
-    jtrans = hhmm(obs.next_transit(ephem_jupiter))
+    jsha = nadeg(2*pi - ephem.degrees(ephem_jupiter.g_ra).norm)
+    jtrans = date2time(obs.next_transit(ephem_jupiter), round2seconds)
     
     obs.date = date
     ephem_saturn.compute(date)
-    satsha = nadeg(2*pi-ephem.degrees(ephem_saturn.g_ra).norm)
-    sattrans = hhmm(obs.next_transit(ephem_saturn))
+    satsha = nadeg(2*pi - ephem.degrees(ephem_saturn.g_ra).norm)
+    sattrans = date2time(obs.next_transit(ephem_saturn), round2seconds)
     
     return [vsha,vtrans,marssha,marstrans,jsha,jtrans,satsha,sattrans,hpmars,hpvenus]
 
@@ -337,7 +370,7 @@ def stellar(date):          # used in starstab
     for line in db.strip().split('\n'):
         st = ephem.readdb(line)
         st.compute(date+0.5)    # calculate at noon
-        out.append([st.name,nadeg(2*pi-ephem.degrees(st.g_ra).norm),nadeg(st.g_dec)])
+        out.append([st.name,nadeg(2*pi - ephem.degrees(st.g_ra).norm),nadeg(st.g_dec)])
     return out
 
 # List of navigational stars with data from Hipparcos, e.g.:
@@ -414,7 +447,7 @@ Markab,f|S|B9,23:04:45.65|60.40,15:12:18.96|-41.30,2.48,2000,0
 # create a list of 'sun above/below horizon' states per Latitude per Normal/Civil/Naut...
 #sunvisible = [[None]*3 for i in range(31)]	# sunvisible[0][0] up to sunvisible[30][2]
 
-def twilight(date, lat, hemisph):   # used in twilighttab (section 1)
+def twilight(date, lat, hemisph, round2seconds = False):   # used in twilighttab (section 1)
     # Returns for given date and latitude(in full degrees):
     # naut. and civil twilight (before sunrise), sunrise, meridian passage, sunset, civil and nautical twilight (after sunset).
     # NOTE: 'twilight' is only called for every third day in the Full Almanac...
@@ -425,62 +458,31 @@ def twilight(date, lat, hemisph):   # used in twilighttab (section 1)
     obs = ephem.Observer()
     latitude = ephem.degrees('{}:00:00.0'.format(lat))
     obs.lat = latitude
-    d = ephem.date(date - 30 * ephem.second)    # search from 30 seconds before midnight
+
+    if round2seconds:
+        d = ephem.date(date) - 0.5 * ephem.second   # search from 0.5 seconds before midnight
+    else:
+        d = ephem.date(date) - 30 * ephem.second    # search from 30 seconds before midnight
+
     obs.date = d
     obs.pressure = 0
     s = ephem.Sun(obs)
     s.compute(d)
     r = s.radius
+    abhd = False                                # above/below horizon display NOT enabled
 
-    obs.horizon = ephem.degrees('-12')+r	# Nautical twilight ...
+    obs.horizon = '-0:34'   # 34' (atmospheric refraction)
     try:
-        out[0] = hhmm(obs.next_rising(s))	# begin
-    except:
-        out[0] = '--:--'
-    obs.date = d
-    try:
-        out[6] = hhmm(obs.next_setting(s))	# end
-    except:
-        out[6] = '--:--'
-    if out[0] == '--:--' and out[6] == '--:--':	# if neither begin nor end...
-        if config.search_next_rising_sun:
-            yn = getsunstate(date, lat, 2)      # ... get the sun state
-        else:
-            yn = midnightsun(mth, hemisph)
-        out[0] = yn
-        out[6] = yn
-#-----------------------------------------------------------
-    obs.horizon = ephem.degrees('-6')+r		# Civil twilight...
-    obs.date = d
-    try:
-        out[1] = hhmm(obs.next_rising(s))	# begin
-    except:
-        out[1] = '--:--'
-    obs.date = d
-    try:
-        out[5] = hhmm(obs.next_setting(s))	# end
-    except:
-        out[5] = '--:--'
-    if out[1] == '--:--' and out[5] == '--:--':	# if neither begin nor end...
-        if config.search_next_rising_sun:
-            yn = getsunstate(date, lat, 1)      # ... get the sun state
-        else:
-            yn = midnightsun(mth, hemisph)
-        out[1] = yn
-        out[5] = yn
-#-----------------------------------------------------------
-    obs.horizon = '-0:34'
-    obs.date = d
-    try:
-        out[2] = hhmm(obs.next_rising(s))	# sunrise
+        out[2] = date2time(obs.next_rising(s), round2seconds)	# sunrise
     except:
         out[2] = '--:--'
     obs.date = d
     try:
-        out[4] = hhmm(obs.next_setting(s))	# sunset
+        out[4] = date2time(obs.next_setting(s), round2seconds)	# sunset
     except:
         out[4] = '--:--'
     if out[2] == '--:--' and out[4] == '--:--':	# if neither sunrise nor sunset...
+        abhd = True                             # enable above/below horizon display
         if config.search_next_rising_sun:
             yn = getsunstate(date, lat, 0)      # ... get the sun state
         else:
@@ -488,20 +490,58 @@ def twilight(date, lat, hemisph):   # used in twilighttab (section 1)
         out[2] = yn
         out[4] = yn
 #-----------------------------------------------------------
+    obs.horizon = ephem.degrees('-6')+r		    # Civil twilight...
     obs.date = d
-    out[3] = hhmm(obs.next_transit(s))
+    try:
+        out[1] = date2time(obs.next_rising(s), round2seconds)	# begin
+    except:
+        out[1] = '--:--'
+    obs.date = d
+    try:
+        out[5] = date2time(obs.next_setting(s), round2seconds)	# end
+    except:
+        out[5] = '--:--'
+    if abhd and out[1] == '--:--' and out[5] == '--:--':	# if neither begin nor end...
+        if config.search_next_rising_sun:
+            yn = getsunstate(date, lat, 1)      # ... get the sun state
+        else:
+            yn = midnightsun(mth, hemisph)
+        out[1] = yn
+        out[5] = yn
+#-----------------------------------------------------------
+    obs.horizon = ephem.degrees('-12')+r	    # Nautical twilight ...
+    obs.date = d
+    try:
+        out[0] = date2time(obs.next_rising(s), round2seconds)	# begin
+    except:
+        out[0] = '--:--'
+    obs.date = d
+    try:
+        out[6] = date2time(obs.next_setting(s), round2seconds)	# end
+    except:
+        out[6] = '--:--'
+    if abhd and out[0] == '--:--' and out[6] == '--:--':	# if neither begin nor end...
+        if config.search_next_rising_sun:
+            yn = getsunstate(date, lat, 2)      # ... get the sun state
+        else:
+            yn = midnightsun(mth, hemisph)
+        out[0] = yn
+        out[6] = yn
+#-----------------------------------------------------------
+    obs.date = d
+    out[3] = date2time(obs.next_transit(s), round2seconds)
     
     return out
 
-##NEW##
 def getsunstate(d, lat, h):
     # populate the sun state (visible or not) for the specified date & latitude
     # note: the first parameter 'd' is an ephem date at midnight
     # note: getsunstate is called when there is neither a sunrise nor a sunset on 'd'
+
     i = config.lat.index(lat)
     latitude = ephem.degrees('{}:00:00.0'.format(lat))
     obs = ephem.Observer()
-    #d = ephem.date(date - 30 * ephem.second)
+    #d = ephem.date(date) - 30 * ephem.second
     obs.pressure = 0
     s = ephem.Sun(obs)
     err = False
@@ -566,14 +606,24 @@ def getsunstate(d, lat, h):
         #print("{}".format(i), nextrising, nextsetting, sunvisible[i][h])
 
     # return the current sunstate
-    out = '--:--'
-    if sunup == True:
-        out = r'\begin{tikzpicture}\draw (0,0) rectangle (12pt,4pt);\end{tikzpicture}'
-    else:
-        out = r'\rule{12Pt}{4Pt}'
-    return out
+    return formatsun('--:--', sunup, True)
 
 ##NEW##
+def formatsun(t, sunup, ab_enabled):
+    # return a formatted string of the current sunrise/sunset/twilight time or state
+    # t = time (hh:mm) or '--:--'
+    # ab_enabled =  True to show time of event or sun event state above/below horizon
+    # ab_enabled =  False to show "event does not happen" as '--:--'
+    # sunup = True  if sun continuously above horizon
+    # sunup = False if sun continuously below horizon
+    
+    if not ab_enabled:      # if above/below horizon display enabled
+        return t
+    elif sunup:
+        return r'''\begin{tikzpicture}\draw (0,0) rectangle (12pt,4pt);\end{tikzpicture}'''
+    else:
+        return r'''\rule{12Pt}{4Pt}'''
+
 def midnightsun(mth, hemisph):
     # simple way to fudge whether the sun is up or down when there's no
     # sunrise or sunset on a day depending on the month and hemisphere only.
@@ -584,11 +634,7 @@ def midnightsun(mth, hemisph):
         sunup = True
     if hemisph == 'S':
         sunup = not(sunup)
-    if sunup == True:
-        out = r'''\begin{tikzpicture}\draw (0,0) rectangle (12pt,4pt);\end{tikzpicture}'''
-    else:
-        out = r'''\rule{12Pt}{4Pt}'''
-    return out
+    return formatsun('--:--', sunup, True)
 
 #-------------------------
 #   MOONRISE/-SET table
@@ -596,14 +642,16 @@ def midnightsun(mth, hemisph):
 
 # create a list of 'moon above/below horizon' states per Latitude...
 #    None = unknown; True = above horizon (visible); False = below horizon (not visible)
-moonvisible = [None] * 31       # moonvisible[0] up to moonvisible[30]
+#    moonvisible[0] is not linked to a latitude but a manual override
+moonvisible = [None] * 32       # moonvisible[0] up to moonvisible[31]
 
-def moonrise_set(date, lat):    # used in twilighttab (section 2)
+def moonrise_set(date, lat):    # used by tables.py in twilighttab (section 2)
+    # - - - TIMES ARE ROUNDED TO MINUTES - - -
     # returns moonrise and moonset for the given date and latitude plus next 2 days:
     #    rise day 1, rise day 2, rise day 3, set day 1, set day 2, set day 3
     # Additionally it also tracks the current state of the moon (above or below horizon)
 
-    i = config.lat.index(lat)
+    i = 1 + config.lat.index(lat)   # index 0 is reserved to enable an explicit setting
     out  = ['--:--','--:--','--:--','--:--','--:--','--:--']	# first event
     out2 = ['--:--','--:--','--:--','--:--','--:--','--:--']	# second event on same day (rare)
 
@@ -611,8 +659,8 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
     latitude = ephem.degrees('{}:00:00.0'.format(lat))
     obs.lat = latitude
     obs.pressure = 0
-    obs.horizon = '-0:34'
-    d = ephem.date(date - 30 * ephem.second)    # search from 30 seconds before midnight
+    obs.horizon = '-0:34'       # 34' (atmospheric refraction)
+    d = ephem.date(date) - 30 * ephem.second    # search from 30 seconds before midnight
     obs.date = d
     m = ephem.Moon(obs)
     m.compute(d)
@@ -622,7 +670,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         firstrising = obs.next_rising(m)
         if firstrising-obs.date >= 1:
             raise ValueError('event next day')
-        out[0] = hhmm(firstrising)		# note: overflow to 00:00 next day is correct here
+        out[0] = date2time(firstrising)		# note: overflow to 00:00 next day is correct here
         lastevent = firstrising
         moonvisible[i] = True
     except Exception:                   # includes NeverUpError and AlwaysUpError
@@ -633,7 +681,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         try:
             nextr = obs.next_rising(m, start=firstrising)
             if nextr-obs.date < 1:
-                out2[0] = hhmm(nextr)   # note: overflow to 00:00 next day is correct here
+                out2[0] = date2time(nextr)   # note: overflow to 00:00 next day is correct here
                 lastevent = nextr
         except UnboundLocalError:
             pass
@@ -650,7 +698,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         firstsetting = obs.next_setting(m)
         if firstsetting-obs.date >= 1:
             raise ValueError('event next day')
-        out[3] = hhmm(firstsetting)		# note: overflow to 00:00 next day is correct here
+        out[3] = date2time(firstsetting)		# note: overflow to 00:00 next day is correct here
         if firstsetting > lastevent:
             lastevent = firstsetting
             moonvisible[i] = False
@@ -661,7 +709,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         try:
             nexts = obs.next_setting(m, start=firstsetting)
             if nexts-obs.date < 1:
-                out2[3] = hhmm(nexts)	# note: overflow to 00:00 next day is correct here
+                out2[3] = date2time(nexts)	# note: overflow to 00:00 next day is correct here
             if nexts > lastevent:
                 moonvisible[i] = False
         except UnboundLocalError:
@@ -679,16 +727,23 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
             getmoonstate(d, lat)			# ...get moon state if unknown
         out[0] = moonstate(i)
         out[3] = moonstate(i)
+
+    if out[0] == '--:--' and out[3] != '--:--':	# if moonset but no moonrise...
+        out[0] = moonset_no_rise(d, date, i, lat)
+
+    if out[0] != '--:--' and out[3] == '--:--':	# if moonrise but no moonset...
+        out[3] = moonrise_no_set(d, date, i, lat)
+
 #-----------------------------------------------------------
     # Moonrise/Moonset on 2nd. day ...
-    d = ephem.date(date + 1 - 30 * ephem.second)
+    d = ephem.date(date + 1) - 30 * ephem.second
     obs.date = d
     m.compute(d)
     try:
         firstrising = obs.next_rising(m)
         if firstrising-obs.date >= 1:
             raise ValueError('event next day')
-        out[1] = hhmm(firstrising)		# note: overflow to 00:00 next day is correct here
+        out[1] = date2time(firstrising)		# note: overflow to 00:00 next day is correct here
         lastevent = firstrising
         moonvisible[i] = True
     except Exception:                   # includes NeverUpError and AlwaysUpError
@@ -699,7 +754,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         try:
             nextr = obs.next_rising(m, start=firstrising)
             if nextr-obs.date < 1:
-                out2[1] = hhmm(nextr)	# note: overflow to 00:00 next day is correct here
+                out2[1] = date2time(nextr)	# note: overflow to 00:00 next day is correct here
                 lastevent = nextr
         except UnboundLocalError:
             pass
@@ -716,7 +771,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         firstsetting = obs.next_setting(m)
         if firstsetting-obs.date >= 1:
             raise ValueError('event next day')
-        out[4] = hhmm(firstsetting)		# note: overflow to 00:00 next day is correct here
+        out[4] = date2time(firstsetting)		# note: overflow to 00:00 next day is correct here
         if firstsetting > lastevent:
             lastevent = firstsetting
             moonvisible[i] = False
@@ -727,7 +782,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         try:
             nexts = obs.next_setting(m, start=firstsetting)
             if nexts-obs.date < 1:
-                out2[4] = hhmm(nexts)	# note: overflow to 00:00 next day is correct here
+                out2[4] = date2time(nexts)	# note: overflow to 00:00 next day is correct here
             if nexts > lastevent:
                 moonvisible[i] = False
         except UnboundLocalError:
@@ -745,16 +800,23 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
             getmoonstate(d, lat)			# ...get moon state if unknown
         out[1] = moonstate(i)
         out[4] = moonstate(i)
+
+    if out[1] == '--:--' and out[4] != '--:--':	# if moonset but no moonrise...
+        out[1] = moonset_no_rise(d, date+1, i, lat)
+
+    if out[1] != '--:--' and out[4] == '--:--':	# if moonrise but no moonset...
+        out[4] = moonrise_no_set(d, date+1, i, lat)
+
 #-----------------------------------------------------------
     # Moonrise/Moonset on 3rd. day ...
-    d = ephem.date(date + 2 - 30 * ephem.second)
+    d = ephem.date(date + 2) - 30 * ephem.second
     obs.date = d
     m.compute(d)
     try:
         firstrising = obs.next_rising(m)
         if firstrising-obs.date >= 1:
             raise ValueError('event next day')
-        out[2] = hhmm(firstrising)		# note: overflow to 00:00 next day is correct here
+        out[2] = date2time(firstrising)		# note: overflow to 00:00 next day is correct here
         lastevent = firstrising
         moonvisible[i] = True
     except Exception:                   # includes NeverUpError and AlwaysUpError
@@ -765,7 +827,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         try:
             nextr = obs.next_rising(m, start=firstrising)
             if nextr-obs.date < 1:
-                out2[2] = hhmm(nextr)	# note: overflow to 00:00 next day is correct here
+                out2[2] = date2time(nextr)	# note: overflow to 00:00 next day is correct here
                 lastevent = nextr
         except UnboundLocalError:
             pass
@@ -782,7 +844,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         firstsetting = obs.next_setting(m)
         if firstsetting-obs.date >= 1:
             raise ValueError('event next day')
-        out[5] = hhmm(firstsetting)		# note: overflow to 00:00 next day is correct here
+        out[5] = date2time(firstsetting)		# note: overflow to 00:00 next day is correct here
         if firstsetting > lastevent:
             lastevent = firstsetting
             moonvisible[i] = False
@@ -793,7 +855,7 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         try:
             nexts = obs.next_setting(m, start=firstsetting)
             if nexts-obs.date < 1:
-                out2[5] = hhmm(nexts)	# note: overflow to 00:00 next day is correct here
+                out2[5] = date2time(nexts)	# note: overflow to 00:00 next day is correct here
             if nexts > lastevent:
                 moonvisible[i] = False
         except UnboundLocalError:
@@ -812,18 +874,38 @@ def moonrise_set(date, lat):    # used in twilighttab (section 2)
         out[2] = moonstate(i)
         out[5] = moonstate(i)
 
+    if out[2] == '--:--' and out[5] != '--:--':	# if moonset but no moonrise...
+        out[2] = moonset_no_rise(d, date+2, i, lat)
+
+    if out[2] != '--:--' and out[5] == '--:--':	# if moonrise but no moonset...
+        out[5] = moonrise_no_set(d, date+2, i, lat)
+
     return out, out2
 
-##NEW##
+def moonstate(ndx):
+    # return the current moonstate (if known)
+    out = '--:--'
+    if moonvisible[ndx] == True:
+        #out = 'UP'
+        #out = r'\framebox(12,4){}'
+        #out = r'{\setlength{\fboxrule}{0.8pt}\setlength{\fboxsep}{0pt}\fbox{\makebox(12,4){}}}'
+        #out = r'{\setlength{\fboxrule}{0.8pt}\fbox{\parbox[c][0pt]{0pt}{ }}}'
+        #out = r'\includegraphics[scale=1.0]{./moonup.jpg}'
+        out = r'\begin{tikzpicture}\draw (0,0) rectangle (12pt,4pt);\end{tikzpicture}'
+    if moonvisible[ndx] == False:
+        #out = 'DOWN'
+        out = r'\rule{12Pt}{4Pt}'
+    return out
+
 def getmoonstate(d, lat):
     # populate the moon state (visible or not) for the specified date & latitude
     # note: the first parameter 'd' is already an ephem date 30 seconds before midnight
     # note: getmoonstate is called when there is neither a moonrise nor a moonset on 'd'
 
-    i = config.lat.index(lat)
+    i = 1 + config.lat.index(lat)   # index 0 is reserved to enable an explicit setting
     latitude = ephem.degrees('{}:00:00.0'.format(lat))
     obs = ephem.Observer()
-    #d = ephem.date(date - 30 * ephem.second)
+    #d = ephem.date(date) - 30 * ephem.second
     obs.pressure = 0
     obs.horizon = '-0:34'
     m = ephem.Moon(obs)
@@ -873,33 +955,291 @@ def getmoonstate(d, lat):
     return
 
 ##NEW##
-def moonstate(ndx):
-    # return the current moonstate (if known)
-    out = '--:--'
-    if moonvisible[ndx] == True:
-        #out = 'UP'
-        #out = r'\framebox(12,4){}'
-        #out = r'{\setlength{\fboxrule}{0.8pt}\setlength{\fboxsep}{0pt}\fbox{\makebox(12,4){}}}'
-        #out = r'{\setlength{\fboxrule}{0.8pt}\fbox{\parbox[c][0pt]{0pt}{ }}}'
-        #out = r'\includegraphics[scale=1.0]{./moonup.jpg}'
-        out = r'\begin{tikzpicture}\draw (0,0) rectangle (12pt,4pt);\end{tikzpicture}'
-    if moonvisible[ndx] == False:
-        #out = 'DOWN'
-        out = r'\rule{12Pt}{4Pt}'
+def moonset_no_rise(d, date, i, lat):
+    # if moonset but no moonrise...
+    msg = ""
+    n = seek_moonrise(d, lat)
+    if n == 1:
+        out = moonstate(i)       # moonrise "below horizon"
+        msg = "below horizon (start)"
+    if n == -1:
+        #print("UP")
+        moonvisible[0] = True
+        out = moonstate(0)       # moonrise "above horizon"
+        msg = "above horizon (end)"
+        #print(out[0])
+    #if msg != "":
+        #print("no moonrise on {} at lat {} => {}".format(ephem.date(date).datetime().strftime("%Y-%m-%d"), lat, msg))
+    if n == 0:
+        out = r'''\raisebox{0.24ex}{\boldmath$\cdot\cdot$~\boldmath$\cdot\cdot$}'''
     return out
 
+##NEW##
+def moonrise_no_set(d, date, i, lat):
+    # if moonrise but no moonset...
+    msg = ""
+    n = seek_moonset(d, lat)
+    if n == 1:
+        out = moonstate(i)       # moonset "above horizon"
+        msg = "above horizon (start)"
+    if n == -1:
+        moonvisible[0] = False
+        out = moonstate(0)       # moonset "below horizon"
+        msg = "below horizon (end)"
+    #if msg != "":
+        #print("no moonset on  {} at lat {} => {}".format(ephem.date(date).datetime().strftime("%Y-%m-%d"), lat, msg))
+    if n == 0:
+        out = r'''\raisebox{0.24ex}{\boldmath$\cdot\cdot$~\boldmath$\cdot\cdot$}'''
+    return out
+
+##NEW##
+def seek_moonset(d, lat):
+    # for the specified date & latitude ...
+    # return -1 if there is NO MOONSET yesterday
+    # return +1 if there is NO MOONSET tomorrow
+    # return  0 if there was a moonset yesterday and will be a moonset tomorrow
+    # note: this is called when there is only a moonrise on the specified date+latitude
+
+    m_set_t = 0     # normal case: assume moonsets yesterday & tomorrow
+
+    i = 1 + config.lat.index(lat)   # index 0 is reserved to enable an explicit setting
+    latitude = ephem.degrees('{}:00:00.0'.format(lat))
+    obs = ephem.Observer()
+    #d = ephem.date(date) - 30 * ephem.second
+    obs.pressure = 0    # turn off PyEphem’s native mechanism for computing atmospheric refraction near the horizon
+    obs.horizon = '-0:34'
+    m = ephem.Moon(obs)
+    err = False
+    obs.date = d
+    obs.lat = latitude
+    m.compute(d)
+    nextsetting = d + 10.0	# in case moonrise but no next moonset
+
+    try:
+        nextsetting = obs.next_setting(m)
+    except ephem.NeverUpError:
+        err = True
+        #print("ns NeverUp")
+        flag_msg("Oops! moon nextS {}: {} occured, line: {}".format(i,sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+    except ephem.AlwaysUpError:
+        err = True
+        m_set_t = +1
+        #print("ns AlwaysUp")
+    except Exception:
+        flag_msg("Oops! moon nextS {}: {} occured, line: {}".format(i,sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+        #sys.exc_clear()		# only in Python 2
+
+    if not(err):	# note - "err == True" *is* expected...
+        # moonset detected - is it after tomorrow?
+        if nextsetting > d + 2.0:
+            m_set_t = +1
+
+    obs.date = d
+    if m_set_t == 0:
+        try:
+            prevsetting = obs.previous_setting(m)
+        except ephem.NeverUpError:
+            err = True
+            m_set_t = -1
+            #print("ps NeverUp")
+        except ephem.AlwaysUpError:
+            err = True
+            #print("ps AlwaysUp")
+            flag_msg("Oops! moon prevS {}: {} occured, line: {}".format(i,sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+        except Exception:
+            flag_msg("Oops! moon prevS {}: {} occured, line: {}".format(i,sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+            #sys.exc_clear()		# only in Python 2
+
+        if not(err):	# note - "err == True" *is* expected...
+            # moonset detected - is it before yesterday?
+            if prevsetting < d - 1.0:
+                m_set_t = -1
+        #print("m_set_t = {}".format(m_set_t))
+    return m_set_t
+
+##NEW##
+def seek_moonrise(d, lat):
+    # return -1 if there is NO MOONRISE yesterday
+    # return +1 if there is NO MOONRISE tomorrow
+    # return  0 if there was a moonrise yesterday and will be a moonrise tomorrow
+    # note: this is called when there is only a moonset on the specified date+latitude
+
+    m_rise_t = 0    # normal case: assume moonrise yesteray & tomorrow
+
+    i = 1 + config.lat.index(lat)   # index 0 is reserved to enable an explicit setting
+    latitude = ephem.degrees('{}:00:00.0'.format(lat))
+    obs = ephem.Observer()
+    #d = ephem.date(date) - 30 * ephem.second
+    obs.pressure = 0
+    obs.horizon = '-0:34'
+    m = ephem.Moon(obs)
+    err = False
+    obs.date = d
+    obs.lat = latitude
+    m.compute(d)
+    nextrising = d + 10.0	# in case moonset but no next moonrise
+
+    try:
+        nextrising  = obs.next_rising(m)
+    except ephem.NeverUpError:
+        err = True
+        m_rise_t = +1
+        #print("nr NeverUp")
+    except ephem.AlwaysUpError:
+        err = True
+        #print("nr AlwaysUp")
+        flag_msg("Oops! moon nextR {}: {} occured, line: {}".format(i,sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+    except Exception:
+        flag_msg("Oops! moon nextR {}: {} occured, line: {}".format(i,sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+        #sys.exc_clear()		# only in Python 2
+
+    if not(err):	# note - "err == True" *is* expected...
+        # moonrise detected - is it after tomorrow?
+        if nextrising > d + 2.0:
+            m_rise_t = +1
+
+    obs.date = d
+    if m_rise_t == 0:
+        try:
+            prevrising = obs.previous_rising(m)
+        except ephem.NeverUpError:
+            err = True
+            #print("pr NeverUp")
+            flag_msg("Oops! moon prevR {}: {} occured, line: {}".format(i,sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+        except ephem.AlwaysUpError:
+            err = True
+            m_rise_t = -1
+            #print("pr AlwaysUp")
+        except Exception:
+            flag_msg("Oops! moon prevR {}: {} occured, line: {}".format(i,sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+            #sys.exc_clear()		# only in Python 2
+
+        if not(err):	# note - "err == True" *is* expected...
+            # moonrise detected - is it before yesterday?
+            if prevrising < d - 1.0:
+                m_rise_t = -1
+
+        #print("m_rise_t = {}".format(m_rise_t))
+    return m_rise_t
+
+#-------------------------
+#   EVENT TIME tables
+#-------------------------
+
+def moonrise_set2(date, lat):    # used by eventtables.py
+    # - - - TIMES ARE ROUNDED TO SECONDS - - -
+    # returns moonrise and moonset for the given date and latitude:
+    #    rise time, set time
+    # Additionally it also tracks the current state of the moon (above or below horizon)
+
+    i = 1 + config.lat.index(lat)   # index 0 is reserved to enable an explicit setting
+    out  = ['--:--','--:--']	# first event
+    out2 = ['--:--','--:--']	# second event on same day (rare)
+
+    obs = ephem.Observer()
+    latitude = ephem.degrees('{}:00:00.0'.format(lat))
+    obs.lat = latitude
+    obs.pressure = 0
+    obs.horizon = '-0:34'       # 34' (atmospheric refraction)
+    d = ephem.date(date) - 0.5 * ephem.second   # search from 0.5 seconds before midnight
+    obs.date = d
+    m = ephem.Moon(obs)
+    m.compute(d)
+#-----------------------------------------------------------
+    # Moonrise/Moonset on the selected day ...
+    try:
+        firstrising = obs.next_rising(m)
+        if firstrising-obs.date >= 1:
+            raise ValueError('event next day')
+        out[0] = hhmmss(firstrising)		# note: overflow to 00:00 next day is correct here
+        lastevent = firstrising
+        moonvisible[i] = True
+    except Exception:                   # includes NeverUpError and AlwaysUpError
+        out[0] = '--:--'
+        lastevent = 0
+
+    if out[0] != '--:--':
+        try:
+            nextr = obs.next_rising(m, start=firstrising)
+            if nextr-obs.date < 1:
+                out2[0] = hhmmss(nextr)   # note: overflow to 00:00 next day is correct here
+                lastevent = nextr
+        except UnboundLocalError:
+            pass
+        except ephem.NeverUpError:
+            pass
+        except ephem.AlwaysUpError:
+            pass
+        except Exception:
+            flag_msg("Oops! {} occured, line: {}".format(sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+            #sys.exc_clear()		# only in Python 2
+
+    obs.date = d
+    try:
+        firstsetting = obs.next_setting(m)
+        if firstsetting-obs.date >= 1:
+            raise ValueError('event next day')
+        out[1] = hhmmss(firstsetting)		# note: overflow to 00:00 next day is correct here
+        if firstsetting > lastevent:
+            lastevent = firstsetting
+            moonvisible[i] = False
+    except Exception:                   # includes NeverUpError and AlwaysUpError
+        out[1] = '--:--'
+
+    if out[1] != '--:--':
+        try:
+            nexts = obs.next_setting(m, start=firstsetting)
+            if nexts-obs.date < 1:
+                out2[1] = hhmmss(nexts)	# note: overflow to 00:00 next day is correct here
+            if nexts > lastevent:
+                moonvisible[i] = False
+        except UnboundLocalError:
+            pass
+        except ephem.NeverUpError:
+            pass
+        except ephem.AlwaysUpError:
+            pass
+        except Exception:
+            flag_msg("Oops! {} occured, line: {}".format(sys.exc_info()[1],sys.exc_info()[2].tb_lineno))
+            #sys.exc_clear()		# only in Python 2
+
+    if out[0] == '--:--' and out[1] == '--:--':	# if neither moonrise nor moonset...
+        if moonvisible[i] == None:
+            getmoonstate(d, lat)			# ...get moon state if unknown
+        out[0] = moonstate(i)
+        out[1] = moonstate(i)
+
+    if out[0] == '--:--' and out[1] != '--:--':	# if moonset but no moonrise...
+        out[0] = moonset_no_rise(d, date, i, lat)
+
+    if out[0] != '--:--' and out[1] == '--:--':	# if moonrise but no moonset...
+        out[1] = moonrise_no_set(d, date, i, lat)
+
+    return out, out2
+
 #------------------------------
 #   Equation of Time section
 #------------------------------
 
-def equation_of_time(date): # used in twilighttab (section 3)
+def equation_of_time(date, round2seconds = False): # used in twilighttab (section 3)
     # returns equation of time, the sun's transit time, 
     # the moon's transit-, antitransit-time, age and percent illumination.
     # (Equation of Time = Mean solar time - Apparent solar time)
 
     py_date = date.tuple()
     py_obsdate = datetime.date(py_date[0], py_date[1], py_date[2])
-    d = ephem.date(date - 30 * ephem.second)
+
+    if round2seconds:
+        # !! transit times are rounded to the nearest second,
+        # !! so the search needs to start and end 0.5 sec earlier
+        # !! e.g. after 23h 59m 59.5s rounds up to 00:00:00 next day
+        d = ephem.date(date) - 0.5 * ephem.second
+    else:
+        # !! transit times are rounded to the nearest minute,
+        # !! so the search needs to start and end 30 sec earlier
+        # !! e.g. after 23h 59m 30s rounds up to 00:00 next day
+        d = ephem.date(date) - 30 * ephem.second
+
     obs = ephem.Observer()
     obs.date = d
     ephem_sun.compute(d)
@@ -910,15 +1250,15 @@ def equation_of_time(date): # used in twilighttab (section 3)
 
     next_s_tr = obs.next_transit(ephem_sun,start=d)
     if next_s_tr - obs.date < 1:
-        transs = hhmm(next_s_tr)
+        transs = date2time(next_s_tr, round2seconds)
 
     next_m_atr = obs.next_antitransit(ephem_moon,start=d)
     if next_m_atr - obs.date < 1:
-        antim = hhmm(next_m_atr)
+        antim = date2time(next_m_atr, round2seconds)
 
     next_m_tr = obs.next_transit(ephem_moon,start=d)
     if next_m_tr - obs.date < 1:
-        transm = hhmm(next_m_tr)
+        transm = date2time(next_m_tr, round2seconds)
 
 #-----------------------------
     obs = ephem.Observer()
@@ -937,12 +1277,12 @@ def equation_of_time(date): # used in twilighttab (section 3)
     eqt00 = ephem.hours(x)
     eqt00 = str(eqt00)[-8:-3]
     if x >= 0:
-        eqt00 = r"\colorbox{{lightgray!80}}{{{}}}".format(eqt00)
+        eqt00 = r"\colorbox{{lightgray!60}}{{{}}}".format(eqt00)
 
     y = round((obs.next_transit(ephem_sun)-(date+0.5))*86400)*2*pi/86400
     eqt12 = ephem.hours(y)
     eqt12 = str(eqt12)[-8:-3]
     if y >= 0:
-        eqt12 = r"\colorbox{{lightgray!80}}{{{}}}".format(eqt12)
+        eqt12 = r"\colorbox{{lightgray!60}}{{{}}}".format(eqt12)
 
     return eqt00,eqt12,transs,transm,antim,age,pct
