@@ -251,7 +251,7 @@ def planetstabm(date):
 
 
 def starstab(date):
-    # returns a table with ephemerieds for the navigational stars
+    # returns a table with ephemerides for the navigational stars
     # OLD: \begin{tabular*}{0.25\textwidth}[t]{@{\extracolsep{\fill}}|rrr|}
 
     if config.tbls == "m":
@@ -977,7 +977,6 @@ def almanac(first_day, pagenum, dfA4):
 % for the title page only...
 \newgeometry{{nomarginpar, top={}, bottom={}, left={}, right={}}}'''.format(tm1,bm1,lm1,rm1)
 
-    dfA4 = dfA4.replace("\\","\\\\")
     alm = alm + r'''
     \begin{{titlepage}}
     \begin{{center}}
